﻿/*
 *  Bitly.Net is a Dot.Net wrapper around the bit.ly RESTful API. bit.ly is an URL shortening service located at http://bit.ly
 *  Copyright (C) 2010 Ckode.dk

 *  This file is part of Bitly.Net.

 *  Bitly.Net is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.

 *  Bitly.Net is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.

 *  You should have received a copy of the GNU Lesser General Public License
 *  along with Bitly.Net.  If not, see <http://www.gnu.org/licenses/>.
 */

using System.Collections.Generic;
using System.Linq;
using Ckode.Bitly.Net.Exceptions;
using Ckode.Bitly.Net.Functions;
using Ckode.Bitly.Net.Request;
using Ckode.Bitly.Net.ResultParsers;
using Ckode.Bitly.Net.Results;
using Ckode.Bitly.Net.Validation;

namespace Ckode.Bitly.Net
{
	public class BitlyClient
	{
		private readonly string login;
		private readonly string apiKey;
		private readonly IValidator validator;

		/// <summary>
		/// The maximum number of shortUrl or hash parameters, to send at once to any function accepting multiple parameters in one call.
		/// </summary>
		public const int MaximumParameters = 15;

		public BitlyClient(string login, string apiKey)
		{
			if (string.IsNullOrEmpty(login))
				throw ExceptionFactory.CreateMissingArgumentException("login");
			this.login = login;

			if (string.IsNullOrEmpty(apiKey))
				throw ExceptionFactory.CreateMissingArgumentException("apiKey");
			this.apiKey = apiKey;

			validator = new Validator();
		}

		#region Functions

		#region Shorten
		public ShortenResult Shorten(string longUrl)
		{
			return Shorten(longUrl, null, null, null);
		}

		public ShortenResult Shorten(string longUrl, string domain)
		{
			return Shorten(longUrl, domain, null, null);
		}

		public ShortenResult Shorten(string longUrl, string xLogin, string xApiKey)
		{
			return Shorten(longUrl, null, xLogin, xApiKey);
		}

		public ShortenResult Shorten(string longUrl, string domain, string xLogin, string xApiKey)
		{
			var function = new ShortenFunction(longUrl, domain, xLogin, xApiKey, validator);

			var rawResult = SendRequest(function);

			var resultParser = new ShortenResultParser();
			return resultParser.Parse(rawResult);
		}
		#endregion

		#region Expand
		public ExpandResult ExpandShortUrl(string shortUrl)
		{
			var function = new ExpandFunction(new[] { shortUrl }, null, validator);

			return ExpandMultiple(function).First();
		}

		public ExpandResult ExpandHash(string hash)
		{
			var function = new ExpandFunction(null, new[] { hash }, validator);

			return ExpandMultiple(function).First();
		}

		public IEnumerable<ExpandResult> ExpandMultiple(IEnumerable<string> shortUrls, IEnumerable<string> hashes)
		{
			var function = new ExpandFunction(shortUrls, hashes, validator);

			return ExpandMultiple(function);
		}

		private IEnumerable<ExpandResult> ExpandMultiple(ExpandFunction function)
		{
			var rawResult = SendRequest(function);

			var resultParser = new ExpandResultParser();
			return resultParser.Parse(rawResult);
		}
		#endregion

		#region Validate

		public ValidateResult Validate(string xLogin, string xApiKey)
		{
			var function = new ValidateFunction(xLogin, xApiKey, validator);

			var rawResult = SendRequest(function);

			var resultParser = new ValidateResultParser();
			return resultParser.Parse(rawResult);
		}
		#endregion

		#region Clicks
		public ClicksResult ClicksByHash(string hash)
		{
			var function = new ClicksFunction(null, new[] { hash }, validator);

			return ClicksSingle(function);
		}

		public ClicksResult ClicksByShortUrl(string shortUrl)
		{
			var function = new ClicksFunction(new[] { shortUrl }, null, validator);

			return ClicksSingle(function);
		}

		public IEnumerable<ClicksResult> ClicksMultiple(IEnumerable<string> shortUrls, IEnumerable<string> hashes)
		{
			var function = new ClicksFunction(shortUrls, hashes, validator);

			return ClicksMultiple(function);
		}

		private ClicksResult ClicksSingle(ClicksFunction function)
		{
			return ClicksMultiple(function).First();
		}

		private IEnumerable<ClicksResult> ClicksMultiple(ClicksFunction function)
		{
			var rawResult = SendRequest(function);

			var resultParser = new ClicksResultParser();
			return resultParser.Parse(rawResult);
		}
		#endregion

		#region Pro Domain
		public ProDomainResult IsProDomain(string domain)
		{
			var function = new ProDomainFunction(domain, validator);

			var rawResult = SendRequest(function);

			var resultParser = new ProDomainResultParser();
			return resultParser.Parse(rawResult);
		}
		#endregion

		#region Lookup
		public LookupResult Lookup(string longUrl)
		{
			var function = new LookupFunction(new[] { longUrl }, validator);

			return Lookup(function).First();
		}
		public IEnumerable<LookupResult> Lookup(IEnumerable<string> longUrls)
		{
			var function = new LookupFunction(longUrls, validator);

			return Lookup(function);
		}

		private IEnumerable<LookupResult> Lookup(LookupFunction function)
		{
			var rawResult = SendRequest(function);

			var resultParser = new LookupResultParser();
			return resultParser.Parse(rawResult);
		}
		#endregion

		#region Authenticate

		public AuthenticateResult Authenticate(string login, string password)
		{
			var function = new AuthenticateFunction(login, password);

			var rawResult = SendRequest(function);

			var resultParser = new AuthenticateResultParser();
			return resultParser.Parse(rawResult);
		}

		#endregion

		#region Info
		public InfoResult InfoByHash(string hash)
		{
			var function = new InfoFunction(null, new[] { hash }, validator);

			return InfoSingle(function);
		}

		public InfoResult InfoByShortUrl(string shortUrl)
		{
			var function = new InfoFunction(new[] { shortUrl }, null, validator);

			return InfoSingle(function);
		}

		public IEnumerable<InfoResult> InfoMultiple(IEnumerable<string> shortUrls, IEnumerable<string> hashes)
		{
			var function = new InfoFunction(shortUrls, hashes, validator);

			return InfoMultiple(function);
		}

		private InfoResult InfoSingle(InfoFunction function)
		{
			return InfoMultiple(function).First();
		}

		private IEnumerable<InfoResult> InfoMultiple(InfoFunction function)
		{
			var rawResult = SendRequest(function);

			var resultParser = new InfoResultParser();
			return resultParser.Parse(rawResult);
		}
		#endregion

		#region Referrers
		public ReferrersResult ReferrersHash(string hash)
		{
			var function = ReferrersFunction.CreateWithHash(hash, validator);
			return Referrers(function);
		}
		public ReferrersResult ReferrersShortUrl(string shortUrl)
		{
			var function = ReferrersFunction.CreateWithShortUrl(shortUrl, validator);
			return Referrers(function);
		}

		private ReferrersResult Referrers(ReferrersFunction function)
		{
			var rawResult = SendRequest(function);

			var resultParser = new ReferrersResultParser();
			return resultParser.Parse(rawResult);
		}
		#endregion

		#endregion

		#region RequestSending

		private string SendRequest(IFunction function)
		{
			var request = function.CreateRequest();
			request.Url = "http://api.bit.ly/v3/" + function.Function.ToString().ToLower();

			AddLoginParameters(request);
			return request.SendRequest();
		}

		private void AddLoginParameters(HttpRequest request)
		{
			request.AddParameter(ParameterNames.Format,  "xml");
			request.AddParameter(ParameterNames.Login, login);
			request.AddParameter(ParameterNames.ApiKey,  apiKey);
		}
		#endregion
	}
}
