﻿/*
 *  Bitly.Net is a Dot.Net wrapper around the bit.ly RESTful API. bit.ly is an URL shortening service located at http://bit.ly
 *  Copyright (C) 2010 Ckode.dk

 *  This file is part of Bitly.Net.

 *  Bitly.Net is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.

 *  Bitly.Net is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.

 *  You should have received a copy of the GNU Lesser General Public License
 *  along with Bitly.Net.  If not, see <http://www.gnu.org/licenses/>.
 */

using System;
using System.Linq;
using Ckode.Bitly.Net.Validation.Validators;
using System.Collections.Generic;

namespace Ckode.Bitly.Net.Exceptions
{
	internal class ExceptionFactory
	{
		public static Exception CreateInvalidShortUrlException(string shortUrl)
		{
			return new ArgumentException(string.Format("shortUrl \"{0}\" is not a valid bit.ly URL (Did you forget the starting protocol, e.g. http:// ?).", shortUrl));
		}

		public static Exception CreateInvalidLongUrlException(string longUrl)
		{
			return new ArgumentException(string.Format("longUrl \"{0}\" is not a valid URL (Did you forget the starting protocol, e.g. http:// ?).", longUrl));
		}

		public static Exception CreateInvalidDomainException(string domain)
		{
			return new ArgumentException(string.Format("Invalid domain \"{0}\", the domain length cannot exceed {1} characters.", domain, DomainValidator.MaximumLength));
		}

		public static Exception CreateInvalidHashException(string hash)
		{
			return new ArgumentException(string.Format("hash \"{0}\" is not a valid bit.ly hash (Did it perhaps include spaces ?).", hash));
		}

		public static Exception CreateEmptyEnumerationException(string parameterName)
		{
			return new ArgumentException(string.Format("{0} sequence contains no elements.", parameterName));
		}

		public static Exception CreateEnumerationTooManyElementsException(string parameterName, int elementCount)
		{
			return new ArgumentException(string.Format("{0} sequence contains too many elements ({1}). It cannot contain more than {2} elements.", parameterName, elementCount, BitlyClient.MaximumParameters));
		}

		public static Exception CreateInvalidXUserException(string xLogin, string xApiKey)
		{
			return new ArgumentException(string.Format("xLogin and xApiKey must either both be empty, or both be non-empty. xLogin: \"{0}\", xApiKey: \"{1}\"", xLogin, xApiKey));
		}

		public static Exception CreateMissingArgumentException(string argumentName)
		{
			return new ArgumentException(string.Format("{0} cannot be null or empty!", argumentName));
		}

		public static Exception CreateHttpErrorException(FunctionName function, string statusCode, string statusText)
		{
			return new BitlyResultException(function, string.Format("HTTP status code was: {0} ({1})", statusCode, statusText));
		}

		public static Exception CreateFunctionErrorWithShortUrl(FunctionName function, string shortUrl, string errorMessage)
		{
			return new BitlyResultException(function, string.Format("Error occurred called with shortUrl \"{0}\". The error was: {1}", shortUrl, errorMessage));
		}

		public static Exception CreateFunctionErrorWithHash(FunctionName function, string hash, string errorMessage)
		{
			return new BitlyResultException(function, string.Format("Error occurred called with hash \"{0}\". The error was: {1}", hash, errorMessage));
		}

		internal static Exception CreateFunctionErrorWithLongUrl(FunctionName function, string longUrl, string errorMessage)
		{
			return new BitlyResultException(function, string.Format("Error occurred called with long url \"{0}\". The error was: {1}", longUrl, errorMessage));
		}

		internal static Exception CreateFunctionErrorWithMissingElements(FunctionName function, string errorMessage)
		{
			return new BitlyResultException(function, string.Format("Error occurred and neither shortUrl, hash or url element was in the result XML. The error was: {0}", errorMessage));
		}

		internal static Exception CreateEmptyMultipleEnumerationException()
		{
			return new ArgumentException("Both shortUrls and hashes are empty");
		}

		internal static Exception CreateElementNotFoundException(FunctionName function, string element, string xml)
		{
			return new BitlyResultException(function,
											string.Format(@"Element ""{0}"" not found in the result XML: {1}", element, xml));
		}

		internal static Exception CreateMultipleElementsNotFoundException(FunctionName function, IEnumerable<string> elements, string xml)
		{
			string elementString = string.Join(", ", elements.ToArray());
			return new BitlyResultException(function,
											string.Format(@"Elements ""{0}"" not found in the result XML: {1}", elementString, xml));
		}
	}
}
