﻿/*
 *  Bitly.Net is a Dot.Net wrapper around the bit.ly RESTful API. bit.ly is an URL shortening service located at http://bit.ly
 *  Copyright (C) 2010 Ckode.dk

 *  This file is part of Bitly.Net.

 *  Bitly.Net is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.

 *  Bitly.Net is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.

 *  You should have received a copy of the GNU Lesser General Public License
 *  along with Bitly.Net.  If not, see <http://www.gnu.org/licenses/>.
 */

using System.Collections.Generic;
using System.Linq;
using Ckode.Bitly.Net.Exceptions;
using Ckode.Bitly.Net.Request;
using Ckode.Bitly.Net.Validation;

namespace Ckode.Bitly.Net.Functions
{
	public enum ParameterType
	{
		ShortUrl = 1,
		Hash = 2
	}

	internal abstract class MultipleParametersFunction : IFunction
	{
		private readonly IEnumerable<string> hashes;
		private readonly IEnumerable<string> shortUrls;

		protected MultipleParametersFunction(IEnumerable<string> shortUrls, IEnumerable<string> hashes, IValidator validator)
		{
			// Handle null parameters gracefully
			shortUrls = shortUrls ?? new string[0];
			hashes = hashes ?? new string[0];

			if (!shortUrls.Any() && !hashes.Any())
				throw ExceptionFactory.CreateEmptyMultipleEnumerationException();

			ValidateCount(shortUrls, "shortUrls");
			ValidateCount(hashes, "hashes");

			this.shortUrls = shortUrls;
			this.hashes = hashes;

			foreach (var shortUrl in shortUrls)
			{
				if (!validator.IsValidShortUrl(shortUrl))
					throw ExceptionFactory.CreateInvalidShortUrlException(shortUrl);
			}

			foreach (var hash in hashes)
			{
				if (!validator.IsValidHash(hash))
					throw ExceptionFactory.CreateInvalidHashException(hash);
			}
		}

		private static void ValidateCount(IEnumerable<string> parameters, string parameterName)
		{
			var elementCount = parameters.Count();
			if (elementCount > BitlyClient.MaximumParameters)
				throw ExceptionFactory.CreateEnumerationTooManyElementsException(parameterName, elementCount);
		}

		protected IEnumerable<KeyValuePair<string, string>> GetParameters()
		{
			foreach (var shortUrl in shortUrls)
				yield return new KeyValuePair<string, string>(ParameterNames.ShortUrl, shortUrl);
			foreach (var hash in hashes)
				yield return new KeyValuePair<string, string>(ParameterNames.Hash, hash);
		}

		public abstract FunctionName Function { get; }

		protected abstract HttpRequest CreateRequestWithoutParameters();

		public HttpRequest CreateRequest()
		{
			var request = CreateRequestWithoutParameters();

			foreach (var shortUrl in shortUrls)
			{
				request.AddParameter(ParameterNames.ShortUrl, shortUrl);
			}
			foreach (var hash in hashes)
			{
				request.AddParameter(ParameterNames.Hash, hash);
			}

			return request;
		}
	}
}
